@@ -200,10 +200,6 @@ int orientation_and_script_detection(STRING& filename,
   ASSERT_HOST(tess->pix_binary() != NULL)
   int width = pixGetWidth(tess->pix_binary());
   int height = pixGetHeight(tess->pix_binary());
-  int resolution = pixGetXRes(tess->pix_binary());
-  // Zero resolution messes up the algorithms, so make sure it is credible.
-  if (resolution < kMinCredibleResolution)
-    resolution = kDefaultResolution;
 
   BLOCK_LIST blocks;
   if (!read_unlv_file(name, width, height, &blocks))
@@ -548,10 +544,10 @@ void ScriptDetector::detect_blob(BLOB_CHOICE_LIST* scores) {
         osr_->scripts_na[i][japanese_id_] += 1.0;
       if (prev_id == hangul_id_)
         osr_->scripts_na[i][korean_id_] += 1.0;
-      if (prev_id == han_id_)
+      if (prev_id == han_id_) {
         osr_->scripts_na[i][korean_id_] += kHanRatioInKorean;
-      if (prev_id == han_id_)
         osr_->scripts_na[i][japanese_id_] += kHanRatioInJapanese;
+      }
     }
   }  // iterate over each orientation
 }
